@@ -26,6 +26,7 @@
 #include "dex-future-set.h"
 #include "dex-gio.h"
 #include "dex-promise.h"
+#include "dex-thread-pool-scheduler.h"
 
 typedef struct _DexFileInfoList DexFileInfoList;
 
@@ -614,7 +615,8 @@ dex_file_query_info_cb (GObject      *object,
 /**
  * dex_file_query_info:
  *
- * Returns: (transfer full): a #DexFuture
+ * Returns: (transfer full): a [class@Dex.Future] that resolves
+ *   to a [class@Gio.FileInfo] or rejects with error.
  */
 DexFuture *
 dex_file_query_info (GFile               *file,
@@ -639,6 +641,58 @@ dex_file_query_info (GFile               *file,
   return DEX_FUTURE (async_pair);
 }
 
+static void
+dex_file_query_file_type_cb (GObject      *object,
+                             GAsyncResult *result,
+                             gpointer      user_data)
+{
+  DexPromise *promise = user_data;
+  GFileInfo *file_info;
+  GError *error = NULL;
+
+  if ((file_info = g_file_query_info_finish (G_FILE (object), result, &error)))
+    {
+      GValue value = G_VALUE_INIT;
+
+      g_value_init (&value, G_TYPE_FILE_TYPE);
+      g_value_set_enum (&value, g_file_info_get_file_type (file_info));
+
+      dex_promise_resolve (promise, &value);
+
+      g_value_unset (&value);
+      g_object_unref (file_info);
+    }
+  else dex_promise_reject (promise, error);
+
+  dex_unref (promise);
+}
+
+/**
+ * dex_file_query_file_type:
+ *
+ * Returns: (transfer full): a [class@Dex.Future] that resolves to a
+ *   [enum@Gio.FileType].
+ */
+DexFuture *
+dex_file_query_file_type (GFile               *file,
+                          GFileQueryInfoFlags  flags,
+                          int                  io_priority)
+{
+  DexPromise *promise;
+
+  dex_return_error_if_fail (G_IS_FILE (file));
+
+  promise = dex_promise_new_cancellable ();
+  g_file_query_info_async (file,
+                           G_FILE_ATTRIBUTE_STANDARD_TYPE,
+                           flags,
+                           io_priority,
+                           dex_promise_get_cancellable (promise),
+                           dex_file_query_file_type_cb,
+                           dex_ref (promise));
+  return DEX_FUTURE (promise);
+}
+
 static void
 dex_file_make_directory_cb (GObject      *object,
                             GAsyncResult *result,
@@ -686,6 +740,77 @@ dex_file_make_directory (GFile *file,
   return DEX_FUTURE (async_pair);
 }
 
+typedef struct
+{
+  GFile *file;
+  DexPromise *promise;
+} MakeDirectory;
+
+static void
+dex_file_make_directory_with_parents_worker (gpointer data)
+{
+  MakeDirectory *state = data;
+  GCancellable *cancellable = dex_promise_get_cancellable (state->promise);
+  GError *error = NULL;
+
+  if (!g_file_make_directory_with_parents (state->file, cancellable, &error))
+    {
+      if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS) &&
+          g_file_query_file_type (state->file, G_FILE_QUERY_INFO_NONE, NULL) == G_FILE_TYPE_DIRECTORY)
+        {
+          g_clear_error (&error);
+          dex_promise_resolve_boolean (state->promise, TRUE);
+        }
+      else
+        {
+          dex_promise_reject (state->promise, g_steal_pointer (&error));
+        }
+    }
+  else
+    {
+      dex_promise_resolve_boolean (state->promise, TRUE);
+    }
+
+  g_clear_object (&state->file);
+  dex_clear (&state->promise);
+  g_free (state);
+}
+
+/**
+ * dex_file_make_directory_with_parents:
+ * @file: a [iface@Gio.File]
+ *
+ * Creates a directory at @file.
+ *
+ * If @file already exists and is a directory, then the future
+ * will resolve to %TRUE.
+ *
+ * Returns: (transfer full): a [class@Dex.Future] that resolves to
+ *   a boolean or rejects with error.
+ *
+ * Since: 0.12
+ */
+DexFuture *
+dex_file_make_directory_with_parents (GFile *file)
+{
+  MakeDirectory *state;
+  DexPromise *promise;
+
+  dex_return_error_if_fail (G_IS_FILE (file));
+
+  promise = dex_promise_new_cancellable ();
+
+  state = g_new0 (MakeDirectory, 1);
+  state->file = g_object_ref (file);
+  state->promise = dex_ref (promise);
+
+  dex_scheduler_push (dex_thread_pool_scheduler_get_default (),
+                      dex_file_make_directory_with_parents_worker,
+                      state);
+
+  return DEX_FUTURE (promise);
+}
+
 static void
 dex_file_enumerate_children_cb (GObject      *object,
                                 GAsyncResult *result,
@@ -755,7 +880,12 @@ dex_file_enumerator_next_files_cb (GObject      *object,
 /**
  * dex_file_enumerator_next_files:
  *
- * Returns: (transfer full): a #DexFuture
+ * Wraps [method@Gio.FileEnumerator.next_files_async].
+ *
+ * Use [method@Dex.Future.await_boxed] to await for the result of this function.
+ *
+ * Returns: (transfer full): a [class@Dex.Future] that resolves to
+ *   a [struct@GLib.List] of [class@Gio.FileInfo]
  */
 DexFuture *
 dex_file_enumerator_next_files (GFileEnumerator *file_enumerator,
@@ -1470,3 +1600,211 @@ dex_file_query_exists (GFile *file)
                            dex_ref (promise));
   return DEX_FUTURE (promise);
 }
+
+static void
+dex_async_initable_init_cb (GObject      *object,
+                            GAsyncResult *result,
+                            gpointer      user_data)
+{
+  DexPromise *promise = user_data;
+  GError *error = NULL;
+
+  if (!g_async_initable_init_finish (G_ASYNC_INITABLE (object), result, &error))
+    dex_promise_reject (promise, g_steal_pointer (&error));
+  else
+    dex_promise_resolve_object (promise, g_object_ref (object));
+
+  dex_unref (promise);
+}
+
+/**
+ * dex_async_initable_init:
+ * @initable: a [iface@Gio.AsyncInitable]
+ * @priority: the priority for the initialization, typically 0
+ *
+ * A helper for g_async_initable_init_async()
+ *
+ * Returns: (transfer full): a [class@Dex.Future] that resolves
+ *   to the @initable instance or rejects with error.
+ *
+ * Since: 0.12
+ */
+DexFuture *
+dex_async_initable_init (GAsyncInitable *initable,
+                         int             priority)
+{
+  DexPromise *promise;
+
+  dex_return_error_if_fail (G_IS_ASYNC_INITABLE (initable));
+
+  promise = dex_promise_new_cancellable ();
+  g_async_initable_init_async (initable,
+                               priority,
+                               dex_promise_get_cancellable (promise),
+                               dex_async_initable_init_cb,
+                               dex_ref (promise));
+  return DEX_FUTURE (promise);
+}
+
+static void
+dex_dbus_connection_close_cb (GObject      *object,
+                              GAsyncResult *result,
+                              gpointer      user_data)
+{
+  DexPromise *promise = user_data;
+  GError *error = NULL;
+
+  if (!g_dbus_connection_close_finish (G_DBUS_CONNECTION (object), result, &error))
+    dex_promise_reject (promise, g_steal_pointer (&error));
+  else
+    dex_promise_resolve_boolean (promise, TRUE);
+
+  dex_unref (promise);
+}
+
+/**
+ * dex_dbus_connection_close:
+ * @connection: a [class@Gio.DBusConnection]
+ *
+ * Asynchronously closes a connection.
+ *
+ * Returns: (transfer full): a [class@Dex.Future] that resolves
+ *   to `true` or rejects with error.
+ *
+ * Since: 0.12
+ */
+DexFuture *
+dex_dbus_connection_close (GDBusConnection *connection)
+{
+  DexPromise *promise;
+
+  dex_return_error_if_fail (G_IS_DBUS_CONNECTION (connection));
+
+  promise = dex_promise_new_cancellable ();
+  g_dbus_connection_close (connection,
+                           dex_promise_get_cancellable (promise),
+                           dex_dbus_connection_close_cb,
+                           dex_ref (promise));
+  return DEX_FUTURE (promise);
+}
+
+static void
+dex_file_set_attributes_cb (GObject      *object,
+                            GAsyncResult *result,
+                            gpointer      user_data)
+{
+  DexPromise *promise = user_data;
+  GFileInfo *file_info = NULL;
+  GError *error = NULL;
+
+  if (!g_file_set_attributes_finish (G_FILE (object), result, &file_info, &error))
+    dex_promise_reject (promise, g_steal_pointer (&error));
+  else
+    dex_promise_resolve_object (promise, g_steal_pointer (&file_info));
+
+  dex_unref (promise);
+}
+
+/**
+ * dex_file_set_attributes:
+ * @file: a [iface@Gio.File]
+ * @file_info: a [class@Gio.FileInfo]
+ * @flags:
+ * @io_priority:
+ *
+ * Returns: (transfer full): a [class@Dex.Future] that resolves to a
+ *   [class@Gio.FileInfo] or rejects with error.
+ *
+ * Since: 0.12
+ */
+DexFuture *
+dex_file_set_attributes (GFile               *file,
+                         GFileInfo           *file_info,
+                         GFileQueryInfoFlags  flags,
+                         int                  io_priority)
+{
+  DexPromise *promise;
+
+  dex_return_error_if_fail (G_IS_FILE (file));
+  dex_return_error_if_fail (G_IS_FILE_INFO (file_info));
+
+  promise = dex_promise_new_cancellable ();
+  g_file_set_attributes_async (file,
+                               file_info,
+                               flags,
+                               io_priority,
+                               dex_promise_get_cancellable (promise),
+                               dex_file_set_attributes_cb,
+                               dex_ref (promise));
+  return DEX_FUTURE (promise);
+}
+
+typedef struct
+{
+  GDestroyNotify notify;
+  gpointer data;
+  DexPromise *promise;
+} FileMove;
+
+static void
+dex_file_move_cb (GObject      *object,
+                  GAsyncResult *result,
+                  gpointer      user_data)
+{
+  FileMove *state = user_data;
+  GError *error = NULL;
+
+  if (!g_file_move_finish (G_FILE (object), result, &error))
+    dex_promise_reject (state->promise, g_steal_pointer (&error));
+  else
+    dex_promise_resolve_boolean (state->promise, TRUE);
+
+  if (state->notify)
+    state->notify (state->data);
+  dex_clear (&state->promise);
+  g_free (state);
+}
+
+/**
+ * dex_file_move:
+ * @progress_callback: (scope notified) (closure progress_callback_data):
+ * @progress_callback_data:
+ * @progress_callback_data_destroy:
+ *
+ * Returns: (transfer full): a [class@Dex.Future] that resolves to %TRUE
+ *   or rejects with error
+ */
+DexFuture *
+dex_file_move (GFile                 *source,
+               GFile                 *destination,
+               GFileCopyFlags         flags,
+               int                    io_priority,
+               GFileProgressCallback  progress_callback,
+               gpointer               progress_callback_data,
+               GDestroyNotify         progress_callback_data_destroy)
+{
+  FileMove *state;
+  DexPromise *promise;
+
+  dex_return_error_if_fail (G_IS_FILE (source));
+  dex_return_error_if_fail (G_IS_FILE (destination));
+
+  promise = dex_promise_new_cancellable ();
+
+  state = g_new0 (FileMove, 1);
+  state->data = progress_callback_data;
+  state->notify = progress_callback_data_destroy;
+  state->promise = dex_ref (promise);
+
+  g_file_move_async (source,
+                     destination,
+                     flags,
+                     io_priority,
+                     dex_promise_get_cancellable (promise),
+                     progress_callback_data,
+                     progress_callback_data_destroy,
+                     dex_file_move_cb,
+                     state);
+
+  return DEX_FUTURE (promise);
+}
